
#include "world.h"

#include "toolbox.h"
#include "tools/toolarrow.h"

#include "samples.h"


static World *world;
static Toolbox *toolbox;


static void init() {
	world = new World(-20, -10, 20, 10, -9.8);
	
	toolbox = new Toolbox(*world, 0, 0, 64);
	(new ToolArrow(*toolbox))->select();
	new Toolbox::Tool(*toolbox, "stick");
	
	
	Samples::chebyshev(*world);
}


static void deinit() {
	delete toolbox;
	delete world;
}


static void draw() {
	bool inputCaptured = false;
	if (!inputCaptured)
		inputCaptured = toolbox->handleButtonsInput();
	
	saveState();
	translate(0.5*worldGetWidth(), 0.5*worldGetHeight());
	scale(1, -1);
	zoom(16);
	strokeWidth(1/8.0);
	
	if (!inputCaptured)
		inputCaptured = toolbox->handleInput();
	world->update(worldGetFrameTime());
	world->draw();
	toolbox->draw();
	
	restoreState();
	
	toolbox->drawButtons();
}


int main() {
	worldSetTitle("Mech2D");
	worldSetSize(800, 600);
	worldSetResizable(TRUE);
	worldSetVariableFrameRate();
	worldSetInit(&init);
	worldSetDeinit(&deinit);
	worldSetDraw(&draw);
	worldRun();
	return 0;
}

