
#include "stick.h"


Stick::Stick(World &world, float x, float y, float l0, float l1, float angle):
	Child(world),
	l0(fabsf(l0)),
	l1(fabsf(l1)),
	length(fabsf(l0) + fabsf(l1)),
	radius(0.5f*world.stickWidth)
{
	b2BodyDef bodyDef;
	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set(x, y);
	bodyDef.angle = angle/180.f*PI;
	body = world.world.CreateBody(&bodyDef);
	
	b2CircleShape circle;
	circle.m_radius = radius;
	
	b2FixtureDef fixtureDef;
	fixtureDef.shape = &circle;
	fixtureDef.density = 1.f;
	fixtureDef.filter.categoryBits = World::CategoryMech;
	fixtureDef.filter.maskBits = World::MaskMech;
	
	circle.m_p.Set(-this->l0, 0.f);
	body->CreateFixture(&fixtureDef);
	circle.m_p.Set( this->l1, 0.f);
	body->CreateFixture(&fixtureDef);
}


Stick::Stick(World &world, float x0, float y0, float x1, float y1):
	Stick(
		world,
		x0, y0,
		0.f,
		sqrtf((x1 - x0)*(x1 - x0) + (y1 - y0)*(y1 - y0)),
		atan2f(y1 - y0, x1 - x0)/PI*180.f )
	{ }


Stick::~Stick() {
	world.world.DestroyBody(body);
}


b2Body* Stick::pickBody(double x, double y) {
	b2Vec2 p = body->GetLocalPoint(b2Vec2(x, y));
	double dx0 = p.x + l0, dx1 = p.x - l1;
	if ( dx0*dx0 + p.y*p.y <= radius*radius
	  || dx1*dx1 + p.y*p.y <= radius*radius
	  || (fabs(p.y) <= radius && p.x >= -l0 && p.x <= l1) )
		return body;
	return nullptr;
}


void Stick::draw() const {
	saveState();
	translate( body->GetPosition().x, body->GetPosition().y );
	rotate( body->GetAngle()/PI*180.0 );
	
	noFill();
	moveTo(0, -radius);
	arcPath( l1 - radius, -radius, 2.0*radius, 2.0*radius, -90.0, 90.0);
	arcPath(-l0 - radius, -radius, 2.0*radius, 2.0*radius, 90.0, 270.0);
	closePath();
	
	//point(-l0, 0);
	//point( l1, 0);
	
	restoreState();
}

