
#include <iostream>
#include "stickstick.h"


static float calcLa(Stick &stickA, Stick &stickB) {
	const b2Transform &ta = stickA.getBody().GetTransform();
	const b2Transform &tb = stickB.getBody().GetTransform();
	
	float d = ta.q.c*tb.q.s - ta.q.s*tb.q.c;
	if (fabs(d) < 1e-5f)
		return 0.f;
	
	float kd = 1.f/d;
	float dpx = tb.p.x - ta.p.x;
	float dpy = tb.p.y - ta.p.y;
	float la = (dpx*tb.q.s - dpy*tb.q.c)*kd;
	if (la < -stickA.l0) la = -stickA.l0;
	if (la >  stickA.l1) la =  stickA.l1;
	return la;
}


StickStick::StickStick(Stick &stickA, Stick &stickB, float la, float lb):
	Revolute(stickA, stickB), stickA(stickA), stickB(stickB), la(la), lb(lb)
{
	b2RevoluteJointDef jointDef = {};
	jointDef.Initialize(&stickA.getBody(), &stickB.getBody(), b2Vec2(0.f, 0.f));
	jointDef.localAnchorA = b2Vec2(la, 0.f);
	jointDef.localAnchorB = b2Vec2(lb, 0.f);
	
	joint = (b2RevoluteJoint*)world.world.CreateJoint(&jointDef);
}


StickStick::StickStick(Stick &stickA, Stick &stickB):
	StickStick(stickA, stickB, calcLa(stickA, stickB), calcLa(stickB, stickA)) { }


StickStick::~StickStick()
	{ world.world.DestroyJoint(joint); }


void StickStick::draw() const {
	const b2Vec2 pa = stickA.getBody().GetWorldPoint(b2Vec2(la, 0.f));
	const b2Vec2 pb = stickB.getBody().GetWorldPoint(b2Vec2(lb, 0.f));
	point(pa.x, pa.y);
	point(pb.x, pb.y);
}
