
#include <math.h>
#include "toolbox.h"



Toolbox::Tool::Tool(Toolbox &toolbox, const std::string &name):
	world(toolbox.world), toolbox(toolbox), name(name)
{
	toolbox.registerTool(*this);
	icon = createAnimation(("data/tools/" + name + ".png").c_str());
}

Toolbox::Tool::~Tool() {
	deselect();
	toolbox.unregisterTool(*this);
	animationDestroy(icon);
}

void Toolbox::Tool::handleButtonClick() {
	if (isEnabled()) select();
}

void Toolbox::Tool::drawButton() const {
	saveState();
	double alpha = isSelected() ? 1
	             : !isEnabled() ? 0.2
	             :                0.5;
	fill(colorByNameA("white", alpha));
	rectTextured(icon, 0, 0, 1, 1);
	restoreState();
}



Toolbox::Toolbox(World &world, double x, double y, double size):
	world(world), x(x), y(y), size(size), selected(), mouseDownTool() { }

Toolbox::~Toolbox() {
	deselect();
	while(!tools.empty())
		delete tools.back();
}


void Toolbox::select(Tool *tool) {
	if (tool == selected) return;
	if (selected) selected->onDeselect();
	selected = tool;
	if (selected) selected->onSelect();
}


bool Toolbox::handleButtonsInput() {
	double mx = transformedMouseX() - x;
	double my = transformedMouseY() - y;
	int index = (int)floor(mx/size);
	Tool *tool = my >= 0 && my < size && index >= 0 && index < (int)tools.size()
	           ? tools[index] : nullptr;
	if (mouseDownTool) {
		if (mouseWentUp("left")) {
			if (mouseDownTool == tool)
				mouseDownTool->handleButtonClick();
			mouseDownTool = nullptr;
		}
		return true;
	}
	
	if (tool && mouseWentDown("left")) {
		mouseDownTool = tool;
		return true;
	}
	
	return false;
}


void Toolbox::drawButtons() const {
	saveState();
	translate(x, y);
	zoom(size);
	noStroke();
	for(List::const_iterator i = tools.begin(); i != tools.end(); ++i)
		{ (*i)->drawButton(); translate(1, 0); }
	restoreState();
}


bool Toolbox::handleInput() {
	if (selected && selected->handleInput())
		return true;
	return false;
}


void Toolbox::draw() const {
	if (selected) selected->draw();
}

