#ifndef TOOLBOX_H
#define TOOLBOX_H


#include <vector>
#include <string>

#include "world.h"


class Toolbox {
public:
	class Tool {
	public:
		World &world;
		Toolbox &toolbox;
		const std::string name;
	
	private:
		Animation icon;
		
	public:
		Tool(Toolbox &toolbox, const std::string &name);
		virtual ~Tool();
		
		Tool(const Tool&) = delete;
		
		virtual bool isEnabled() const { return true; }
		virtual void onSelect() { }
		virtual void onDeselect() { }
		
		virtual void handleButtonClick();
		virtual void drawButton() const;
		
		virtual bool handleInput() { return false; }
		virtual void draw() const { }
		
		bool isSelected() const { return toolbox.getSelected() == this; }
		void select() { toolbox.select(this); }
		void deselect() { if (isSelected()) toolbox.deselect(); }
	};
	
	typedef std::vector<Tool*> List;

public:
	World &world;
	const double x;
	const double y;
	const double size;
	
private:
	List tools;
	Tool *selected;
	Tool *mouseDownTool;
	
	void registerTool(Tool &tool)
		{ tools.push_back(&tool); }
	void unregisterTool(Tool &tool) {
		for(List::reverse_iterator ri = tools.rbegin(); ri != tools.rend(); ++ri)
			if (*ri == &tool) { tools.erase((++ri).base()); return; }
	}
	
public:
	Toolbox(World &world, double x, double y, double size);
	~Toolbox();
	
	Toolbox(const Toolbox&) = delete;
	Toolbox& operator=(const Toolbox&) = delete;
	
	Tool* getSelected() const { return selected; }
	void select(Tool *tool);
	void deselect() { select(nullptr); }
	
	bool handleButtonsInput();
	void drawButtons() const;
	
	bool handleInput();
	void draw() const;
};


#endif
