
#include <algorithm>

#include "world.h"


World::World(float minx, float miny, float maxx, float maxy, float gravity):
	minx(std::min(minx, maxx)),
	miny(std::min(miny, maxy)),
	maxx(std::max(minx, maxx)),
	maxy(std::max(miny, maxy)),
	gravity(gravity),
	stickWidth(0.5f),
	motorWidth(2.f*stickWidth),
	world(b2Vec2(0.f, gravity)),
	remainingTime()
{
	const float x = 0.5f*(this->minx + this->maxx);
	const float y = 0.5f*(this->miny + this->maxy);
	const float w = this->maxx - this->minx;
	const float h = this->maxy - this->miny;
	const float gw = 0.5f*std::min(w, h);
	const float hh = 0.5f*h;
	const float hw = 0.5f*w;
	const float hgw = 0.5f*gw;
	
	b2BodyDef grounBodyDef;
	grounBodyDef.position.Set(x, y);
	b2Body *groundBody = world.CreateBody(&grounBodyDef);
	
	b2PolygonShape groundBox;
	b2FixtureDef fixtureDef;
	fixtureDef.shape = &groundBox;
	fixtureDef.filter.categoryBits = World::CategoryGround;
	fixtureDef.filter.maskBits = World::MaskGround;
	
	groundBox.SetAsBox(hw + gw, hgw, b2Vec2(0.f, -hh - hgw), 0.f);
	groundBody->CreateFixture(&fixtureDef);
	groundBox.SetAsBox(hw + gw, hgw, b2Vec2(0.f,  hh + hgw), 0.f);
	groundBody->CreateFixture(&fixtureDef);
	groundBox.SetAsBox(hgw, hh + gw, b2Vec2(-hw - hgw, 0.f), 0.f);
	groundBody->CreateFixture(&fixtureDef);
	groundBox.SetAsBox(hgw, hh + gw, b2Vec2( hw + hgw, 0.f), 0.f);
	groundBody->CreateFixture(&fixtureDef);
}


World::~World() {
	while(!children.empty()) delete children.back();
}


void World::update(float dt) {
	const float timeStep = 1.f/60.f;
	remainingTime += dt;
	while(remainingTime >= 0.5*timeStep) {
		world.Step(timeStep, 6, 2);
		remainingTime -= timeStep;
	}
}


void World::draw() const {
	saveState();
	for(ChildrenList::const_iterator i = children.begin(); i != children.end(); ++i)
		(*i)->draw();
	noFill();
	rect(minx, miny, maxx - minx, maxy - miny);
	restoreState();
}

